//---------------------------------------------------------------------------

#include <vcl.h>
#include <math.h>
#pragma hdrstop

#include "Unit1.h"
#include "Unit2.h"
#include "h2ofunc.h"
#include "meohfunc.h"
#include "co2func.h"
#include "func.h"
#include "Unit3.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;
//extern parameters
double R,kB=1.3806e-23,NA=6.0221367e23,	//universal parameters

		Tc,dc,pc,	//critical paramters
        p0,T0,d0,	//input (or unknown) parameters
		Mw,			//atomic weight
        Ts,ds,ps,	//scaling parameters (often same as critical parameters)
        Tr,			//inverse of scaled temperature (Tr=Ts/T0)
        dr,			//scaled density
        pr,			//scaled density
					//("scaled" does not mean "reduced"!)

		Sp,	//saturation pressure
		Sdv,	//saturation density in vapor
        Sdl,	//saturation density in liquid
        Mp;		//melting pressure


int varsel;	//input parameter selection (0=PRES, 1=DENS, defined)

//function set for calculation
TFuncs Fmeltp,		//for melting pressure on T
		FrangeT;	//for limited to the range in T
TFuncr FrangePT;	//for limited to the range in T and p
TFuncst FSatu;		//for calculating saturation parameters on T
TFuncz FZ;			//for calculating Z: compressibity factor
TFuncx FX;			//for calculating almost all parameters
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
	: TForm(Owner)
{
	Form1->Left=Screen->Width/2-Form1->Width/2;
	Form1->Top=Screen->Height/2-Form1->Height/2;

	ComboBoxT->ItemIndex=0;
	ComboBoxP->ItemIndex=0;
	ComboBoxRho->ItemIndex=0;
	ComboBoxUnit->ItemIndex=0;
    Rho_on=true;
    unitT=unitP=unitRho=unitM=0;

    asT[0]="K";
    asT[1]="degC";
    asT[2]="Tr";

    asP[0]="MPa";
    asP[1]="Pa";
    asP[2]="bar";
    asP[3]="atm";
    asP[4]="Pr";

    asRho[0]="kg / m3";
    asRho[1]="g / cm3";
    asRho[2]="mol / l";
    asRho[3]="molec / A3";
    asRho[4]="rho-r";

    asV[0]="m3 / kg";
    asV[1]="cm3 / g";
    asV[2]="l / mol";
    asV[3]="A3 / molec";
    asV[4]="Vr";

	RadioGroupMatter->OnClick(this);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::RadioGroupMatterClick(TObject *Sender)
{
    if(RadioGroupMatter->ItemIndex==0){	//H2O
    	Mw=18.015257;
		R=8.31451,
        Tc=647.096;
    	pc=22.064e6;
    	dc=322/Mw*1e3;
        Ts=ds=ps=1;	//not scaled

		Fmeltp=fwmeltp;
        FrangeT=fwrangeT;
		FrangePT=fwrangePT;
		FZ=fwZ;
		FX=fwX;
		FSatu=fwsatu;
    }
    if(RadioGroupMatter->ItemIndex==1){	//MeOH
        Mw=32.04216;
		R=8.31448,
    	Tc=512.60;
    	pc=8.1035e6;
    	dc=8.60e3; 	//mol m-3 	//8.60e-3 mol cm^-3

    	Ts=513.380;
    	ds=8.78517e3;
        ps=1;
        pr=0;	//ps and pr do not use, these are set temporary.

		Fmeltp=fmmeltp;
        FrangeT=fmrangeT;
		FrangePT=fmrangePT;
		FZ=fmZ;
		FX=fmX;
		FSatu=fmsatu;
	}
    if(RadioGroupMatter->ItemIndex==2){	//CO2
        Mw=44.0098;
		R=8.31451,
    	Tc=304.1282;
    	pc=7.3773e6;
    	dc=467.6/Mw*1e3; 	//mol m-3

    	Ts=Tc;
    	ds=dc;
        ps=1;
        pr=0;	//ps and pr do not use, these are set temporary.

		Fmeltp=fcmeltp;
        FrangeT=fcrangeT;
		FrangePT=fcrangePT;
		FZ=fcZ;
		FX=fcX;
		FSatu=fcsatu;
	}
/*
    if(RadioGroupMatter->ItemIndex==3){	//Xe
        Mw=131.30;
		R=8.3143,
    	Tc=289.734;
    	pc=5.840e6;
    	dc=1099/Mw*1e3; 	//mol m-3

    	Ts=Tc;
    	ds=1000/Mw*1e3;
        ps=1;
        pr=0;	//ps and pr do not use, these are set temporary.

		Fmeltp=fxmeltp;
        FrangeT=fxrangeT;
		FrangePT=fxrangePT;
		FZ=fxZ;
		FX=fxX;
		FSatu=fxsatu;
	}
*/

}
//---------------------------------------------------------------------------
void __fastcall TForm1::ButtonCalcClick(TObject *Sender)
{
	EditMes->Text="";
    EditF->Text="";
    EditU->Text="";
    EditH->Text="";
    EditG->Text="";
    EditS->Text="";
    EditCv->Text="";
    EditCp->Text="";
    EditZ->Text="";
	EditFP->Text="";
	EditW->Text="";
    EditJT->Text="";
    EditC2->Text="";
    EditEps->Text="";
    EditVisc->Text="";
    EditTherc->Text="";
    EditDth->Text="";
    EditKw->Text="";
    Editn->Text="";
    Editalpha->Text="";
    Editkappa->Text="";

    EditmeltP->Text="";
    EditsatuP->Text="";
    EditsatuRhov->Text="";
    EditsatuRhol->Text="";

    varsel=RadioGroupVar->ItemIndex;

    T0=unit2stdT(EditT->Text.ToDouble());		//K
    if(varsel==PRES){
    	p0=unit2stdP(EditP->Text.ToDouble());	//Pa
    	d0=0;
        EditRho->Text="";
    }
    if(varsel==DENS){
    	d0=unit2stdRho(EditRho->Text.ToDouble());	//mol/m3
    	p0=0;
        EditP->Text="";
    }


	AnsiString aSp,aSdv,aSdl,aMp,asx,asmes="";

    Tr=Ts/T0;

    asx=FrangeT(T0);
    asmes+=asx;
    if(asx!="") goto err;

    aMp=Fmeltp(Tr);
    Mp=atof(aMp.c_str());
		//cf. Fmeltp returns the value of Mp in stdunit.
    if(Mp==0) EditmeltP->Text=aMp;
	else EditmeltP->Text=std2unitP(Mp);


    FSatu(Tr,&aSp,&aSdv,&aSdl);
    	//cf. FSatu returns the values of Sp, Sdv, and Sdl in stdunit.

    Sp=atof(aSp.c_str());
    Sdv=atof(aSdv.c_str());
    Sdl=atof(aSdl.c_str());

    if(Sp==0) EditsatuP->Text=aSp;
	else EditsatuP->Text=std2unitP(Sp);
    if(Sdv==0) EditsatuRhov->Text=aSdv;
	else EditsatuRhov->Text=std2unitRho(Sdv);
    if(Sdl==0) EditsatuRhol->Text=aSdl;
	else EditsatuRhol->Text=std2unitRho(Sdl);

    if(varsel==PRES){
    	asx=FrangePT(p0,T0);
        asmes+=asx;
        if(asx!="") goto err;

        pr=p0/ps;
        d0=p02d0(T0,p0,Sp,Sdv,Sdl);
        EditRho->Text=std2unitRho(d0);
    }
    dr=d0/ds;

    if(Sp!=0&&d0>=Sdv&&d0<=Sdl){
    	asmes+="rho may be in the co-existence region (meta-stable?);";
		if(d0>=1.05*Sdv&&d0<=0.9*Sdl) goto err;
	}

    if(varsel==DENS){
        p0=d02p0(T0,d0,Sp,Sdv,Sdl);
        pr=p0/ps;
        EditP->Text=std2unitP(p0);

        asx=FrangePT(p0,T0);
        asmes+=asx;
        if(asx!="") goto err;
    }

    double F,U,H,G,S,Cv,Cp,Z,FP,W,JT,C2,Ap,kT;

    FX(dr,Tr,pr,&F,&U,&H,&G,&S,&Cv,&Cp,&Z,&FP,&W,&JT,&C2,&Ap,&kT);

    EditF->Text=std2unitE(F);
    EditU->Text=std2unitE(U);
    EditH->Text=std2unitE(H);
    EditG->Text=std2unitE(G);
    EditS->Text=std2unitE(S);
    EditCv->Text=std2unitE(Cv);
    EditCp->Text=std2unitE(Cp);
    EditZ->Text=ASDigit(Z);
	EditFP->Text=ASDigit(FP);
	EditW->Text=ASDigit(W);
    EditJT->Text=std2unitJT(JT);
    EditC2->Text=std2unitC2(C2);
    Editalpha->Text=ASDigit(Ap);
    Editkappa->Text=std2unitJT(kT);


err:
	EditMes->Text=asmes;

}
//---------------------------------------------------------------------------
double TForm1::p02d0(double T0x,double p0x,double Spx,double Sdvx,double Sdlx)
{

	if(RadioGroupMatter->ItemIndex==0){
//in case that basic function is Gibbs
        int fws=fwselect(0,Ts/T0x,p0x,Spx,Sdvx,Sdlx);
		if(fws==1||fws==2||fws==5) return p_invZRT(T0x,p0x);
	}

//in case that basic function is Helmhortz

	double drx=(1.0-ALM0)*dc,ddr=dc/2,px;

    if(Spx!=0){	//Above CP, Spx == 0
    	if(p0x<Spx){
           	drx=Sdvx*0.51;
            ddr=drx*0.5;
        }
        if(p0x>Spx){
        	drx=Sdlx*1.4;
            ddr=Sdlx*0.25;
        }
    }
    drx/=ds;
    ddr/=ds;

    if(!(Spx!=0&&p0x<Spx)){
    	px=ZrhoRT(T0x,drx+0.1*dc/ds);
        while(px<p0x){
        	drx+=0.1*dc/ds;
            px=ZrhoRT(T0x,drx+0.1*dc/ds);
        }
    }

    do{
    	px=ZrhoRT(T0x,drx);
        if(px>p0x) drx-=ddr;
        else drx+=ddr;
        while(drx<=0) drx=(drx+ddr)*0.5;
        ddr*=0.5;

    }while(ddr/drx>=ALM0);

    return drx*ds;
}
//---------------------------------------------------------------------------
double TForm1::d02p0(double T0x,double d0x,double Spx,double Sdvx,double Sdlx)
{
	if(RadioGroupMatter->ItemIndex==0){
        int fws=fwselect(d0x,Ts/T0x,0,Spx,Sdvx,Sdlx);

		if(fws!=3){
	//in case that basic function is Gibbs
			double prx=pc,dpr=pc/2,dx;

            if(fws==2){
            	prx=0.5*fw3brdp0(1.0/T0x);
                dpr=0.5*prx;
            }

	    	if(Spx!=0){	//Above CP, Sdv and Sdl == 0
	    		if(d0x<Sdvx){
	    	       	prx=Spx*0.51;
	    	        dpr=prx*0.5;
	    	    }
	    	    if(d0x>Sdlx){
	    	    	prx=Spx*1.4;
	    	        dpr=Spx*0.25;
	    	    }
	    	}

			dx=p_invZRT(T0x,prx+0.1*pc/ps);
   			while(dx<d0x&&dx!=0){
   				prx+=0.1*pc/ps;
	       	    dx=p_invZRT(T0x,prx+0.1*pc/ps);
                if(dx<0) prx-=0.125*pc/ps;
           	}


            double dxdd,dxud;
	    	do{
	    		dx=p_invZRT(T0x,prx);

	    		dxdd=fabs(p_invZRT(T0x,prx-dpr)-d0x),
                dxud=fabs(p_invZRT(T0x,prx+dpr)-d0x);

                if(fabs(dx-d0x)>min(dxdd,dxud)){
					if(dxdd<dxud) prx-=dpr;
                    else prx+=dpr;
                }
	    	    dpr*=0.5;
	    	}while(dpr>=ALM0*prx);

            dx=p_invZRT(T0x,prx);


            int i=0;
            while(fabs(dx-d0x)>d0x*ALM0&&i++<30){
                dpr=0.5*prx;

	    	do{
	    		dx=p_invZRT(T0x,prx);

	    		dxdd=fabs(p_invZRT(T0x,prx-dpr)-d0x),
                dxud=fabs(p_invZRT(T0x,prx+dpr)-d0x);

                if(fabs(dx-d0x)>min(dxdd,dxud)){
					if(dxdd<dxud) prx-=dpr;
                    else prx+=dpr;
                }
	    	    dpr*=0.5;
	    	}while(dpr>=ALM0*prx);

            }

            return prx;
        }
	}

	//in case that basic function is Helmhortz
	return ZrhoRT(T0x,d0x/ds);
}
//---------------------------------------------------------------------------
double TForm1::ZrhoRT(double T0x,double drx)
{
	return drx*ds*R*T0x*FZ(drx,Ts/T0x,0);
}
//---------------------------------------------------------------------------
double TForm1::p_invZRT(double T0x,double p0x)
{
	return p0x/(FZ(0,Ts/T0x,p0x/ps)*R*T0x);
}
//---------------------------------------------------------------------------
double TForm1::unit2stdT(double xt)
{
	double x=xt;
	if(unitT==1) x+=273.15;
	if(unitT==2) x*=Tc;
    return x;
}
//---------------------------------------------------------------------------
double TForm1::unit2stdP(double xp)
{
	double u2sP[5]={1e6,1.0,1e5,1.01325e5,pc};
	return xp*u2sP[unitP];	//... -> Pa
}
//---------------------------------------------------------------------------
double TForm1::unit2stdRho(double xd)
{
	double u2sRho[5]={1e3/Mw,1e6/Mw,1e3,1e30/NA,dc};
	if(Rho_on) return xd*u2sRho[unitRho];	//... -> mol/m3
	else return u2sRho[unitRho]/xd;		//... -> m3/mol	-> mol/m3
}
//---------------------------------------------------------------------------
AnsiString TForm1::std2unitT(double xt)
{
	double x=xt;		//K -> ...
	if(unitT==1) x-=273.15;
    if(unitT==2) x/=Tc;
	return ASDigit(x);
}
//---------------------------------------------------------------------------
AnsiString TForm1::std2unitP(double xp)
{
	double u2sP[5]={1e6,1.0,1e5,1.01325e5,pc};
	return ASDigit(xp/u2sP[unitP]);  //Pa -> ...
}
//---------------------------------------------------------------------------
AnsiString TForm1::std2unitRho(double xd)
{
	double u2sRho[5]={1e3/Mw,1e6/Mw,1e3,1e30/NA,dc};
	if(Rho_on) return ASDigit(xd/u2sRho[unitRho]);
        //mol/m3 -> ...
	else return ASDigit(u2sRho[unitRho]/xd);
    	//mol/m3 -> m3/mol -> ...
}
//---------------------------------------------------------------------------
AnsiString TForm1::std2unitE(double xe)
{
	double x=xe;		//J/mol
	if(unitM==1) x/=Mw/1000;
    if(unitM==2) x/=R;
	return ASDigit(x);
}
//---------------------------------------------------------------------------
AnsiString TForm1::std2unitJT(double xJT)
{
	double s2uP[5]={1e6,1.0,1e5,1.01325e5,pc};
	return ASDigit(xJT*s2uP[unitP]);	//K/Pa -> ...
}
//---------------------------------------------------------------------------
AnsiString TForm1::std2unitC2(double xC2)
{
	double s2uRho[5]={1e3/Mw,1e6/Mw,1e3,1e30/NA,dc};
	return ASDigit(xC2*s2uRho[unitRho]);	//m3/mol -> ...
}
//---------------------------------------------------------------------------
void __fastcall TForm1::ComboBoxTChange(TObject *Sender)
{
	if(ComboBoxT->ItemIndex!=unitT){
		double x=EditT->Text.ToDouble();
		if(unitT==1) x+=273.15;
		if(unitT==2) x*=Tc;

		unitT=ComboBoxT->ItemIndex;

		if(unitT==1) x-=273.15;
		if(unitT==2) x/=Tc;
        EditT->Text=ASDigit(x);
    }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::ComboBoxPChange(TObject *Sender)
{

	if(ComboBoxP->ItemIndex!=unitP){
        int unitPn=ComboBoxP->ItemIndex;
		double u2s[5]={1e6,1.0,1e5,1.01325e5,pc};

		double uvu=u2s[unitP]/u2s[unitPn];

		double x=EditP->Text.ToDouble();
        EditP->Text=ASDigit(x*uvu);

		double xm=atof(EditmeltP->Text.c_str());
	    if(xm!=0) EditmeltP->Text=ASDigit(xm*uvu);

		double xs=atof(EditsatuP->Text.c_str());
	    if(xs!=0) EditsatuP->Text=ASDigit(xs*uvu);

		double xJT=atof(EditJT->Text.c_str());	//Joule-Thomson unit in K/MPa
	    if(xJT!=0) EditJT->Text=ASDigit(xJT*uvu);

		double xkT=atof(Editkappa->Text.c_str());	//compression coef. unit in /MPa
	    if(xkT!=0) Editkappa->Text=ASDigit(xkT*uvu);

        LabelmeltP->Caption=asP[unitPn];
        LabelsatuP->Caption=asP[unitPn];
		LabelJT->Caption="K / "+asP[unitPn];
		Labelkappa->Caption="/ "+asP[unitPn];

        unitP=unitPn;
    }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::ComboBoxRhoChange(TObject *Sender)
{

	if(ComboBoxRho->ItemIndex!=unitRho){
		double u2s[5]={1e3/Mw,1e6/Mw,1e3,1e30/NA,dc};

		int unitRhon=ComboBoxRho->ItemIndex;

		double x=atof(EditRho->Text.c_str()),
        		uvu=u2s[unitRho]/u2s[unitRhon];

		double xv=atof(EditsatuRhov->Text.c_str());
		double xl=atof(EditsatuRhol->Text.c_str());
		double xC2=atof(EditC2->Text.c_str());	//2nd Virial Coef. in unit m-3/kg

        if(Rho_on){
        	EditRho->Text=ASDigit(x*uvu);
            if(xv!=0)EditsatuRhov->Text=ASDigit(xv*uvu);
            if(xl!=0)EditsatuRhol->Text=ASDigit(xl*uvu);
            LabelsatuRhov->Caption=asRho[unitRhon];
            LabelsatuRhol->Caption=asRho[unitRhon];
        }else{
        	EditRho->Text=ASDigit(x/uvu);
            if(xv!=0)EditsatuRhov->Text=ASDigit(xv/uvu);
            if(xl!=0)EditsatuRhol->Text=ASDigit(xl/uvu);
            LabelsatuRhov->Caption=asV[unitRhon];
            LabelsatuRhol->Caption=asV[unitRhon];
        }
        if(xC2!=0) EditC2->Text=ASDigit(xC2/uvu);



        LabelC2->Caption=asV[unitRhon];

        unitRho=unitRhon;
    }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::ComboBoxUnitChange(TObject *Sender)
{
	if(ComboBoxUnit->ItemIndex!=unitM){
        int unitMn=ComboBoxUnit->ItemIndex;

		double xu=atof(EditU->Text.c_str()),
		xh=atof(EditH->Text.c_str()),
		xf=atof(EditF->Text.c_str()),
		xg=atof(EditG->Text.c_str()),
		xs=atof(EditS->Text.c_str()),
		xcv=atof(EditCv->Text.c_str()),
		xcp=atof(EditCp->Text.c_str());

 		double u2s[3]={1,Mw*1e-3,R};

		double uvu=u2s[unitM]/u2s[unitMn];

        EditU->Text=ASDigit(xu*uvu);
        EditH->Text=ASDigit(xh*uvu);
        EditF->Text=ASDigit(xf*uvu);
        EditG->Text=ASDigit(xg*uvu);
        EditS->Text=ASDigit(xs*uvu);
        EditCv->Text=ASDigit(xcv*uvu);
        EditCp->Text=ASDigit(xcp*uvu);

		AnsiString asUM[3]={"J / mol","J / kg","kB / molec"},
		asUM2[3]={"J / (mol K)","J / (kg K)","kB / (molec K)"};

        LabelU->Caption=asUM[unitMn];
        LabelH->Caption=asUM[unitMn];
        LabelF->Caption=asUM[unitMn];
        LabelG->Caption=asUM[unitMn];

        LabelS->Caption=asUM2[unitMn];
        LabelCv->Caption=asUM2[unitMn];
        LabelCp->Caption=asUM2[unitMn];

        unitM=unitMn;
    }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Button1Click(TObject *Sender)
{
	Form2->Show();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::RadioGroupVarClick(TObject *Sender)
{
	if(RadioGroupVar->ItemIndex==0){
		EditP->Color=clWindow;
		EditP->ReadOnly=false;
		EditRho->Color=clBtnFace;
		EditRho->ReadOnly=true;
    }
	if(RadioGroupVar->ItemIndex==1){
		EditP->Color=clBtnFace;
		EditP->ReadOnly=true;
		EditRho->Color=clWindow;
		EditRho->ReadOnly=false;
    }

}
//---------------------------------------------------------------------------
void __fastcall TForm1::ButtonSetCpClick(TObject *Sender)
{
	EditT->Text=std2unitT(Tc);
	EditP->Text=std2unitP(pc);
	EditRho->Text=std2unitRho(dc);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::ButtonXRhoVClick(TObject *Sender)
{
	if(Rho_on){
		Rho_on=false;

		RadioGroupVar->Items->Strings[1]="V=";
		for(int i=0;i<5;i++) ComboBoxRho->Items->Strings[i]=asV[i];

        LabelsatuRhov->Caption=asV[unitRho];
        LabelsatuRhol->Caption=asV[unitRho];
        LabelsatuRhovt->Caption="V-gas =";
        LabelsatuRholt->Caption="V-liq =";
        ComboBoxRho->Text=asV[unitRho];


        Form2->ComboBoxR->Items->Strings[2]="V";
        Form2->ComboBoxC->Items->Strings[2]="V";
        Form2->ComboBoxQ->Items->Strings[1]="Specific Volume: V";

    }else{
    	Rho_on=true;

    	RadioGroupVar->Items->Strings[1]="rho=";
		for(int i=0;i<5;i++) ComboBoxRho->Items->Strings[i]=asRho[i];
        LabelsatuRhov->Caption=asRho[unitRho];
        LabelsatuRhol->Caption=asRho[unitRho];
        LabelsatuRhovt->Caption="rho-gas =";
        LabelsatuRholt->Caption="rho-liq =";
        ComboBoxRho->Text=asRho[unitRho];

        Form2->ComboBoxR->Items->Strings[2]="rho";
        Form2->ComboBoxC->Items->Strings[2]="rho";
        Form2->ComboBoxQ->Items->Strings[1]="Density: rho";

	}

    EditRho->Text=ASDigit(1.0/EditRho->Text.ToDouble());
    double xv=atof(EditsatuRhov->Text.c_str());
    if(xv!=0) EditsatuRhov->Text=ASDigit(1.0/xv);
    double xl=atof(EditsatuRhol->Text.c_str());
    if(xl!=0) EditsatuRhol->Text=ASDigit(1.0/xl);

}
//---------------------------------------------------------------------------
void __fastcall TForm1::ButtonAboutClick(TObject *Sender)
{
	Form3->Show();
}
//---------------------------------------------------------------------------
AnsiString ASDigit(double xd)
{
    return FloatToStrF(xd,ffGeneral,8,4);
        //digit=8,exp=4, to see help of FloatToStrF
}
//---------------------------------------------------------------------------



