#include <math.h>
#include "Unit1.h"
#include "xefunc.h"
#include "func.h"

/*
The reference of EOS of Xe is different in abbreviation and formlization.
Large different points are:

Normalized Helmhortz energy phi'=F/RTr, whereas phi=F/RT in others.
And phi' is described as a function of T/Tr and rho/rho-r(=del),
whereas phi as that of Tr/T(=tau) and del.


Therefore, phi=phi*(Tr/T)
If phi'=f(T/Tr,del), phi(tau,del)=Tr/T*phi'=tau*f(1.0/tau,del).

We use phi(tau,del), of course.
*/
//Xe func
//---------------------------------------------------------------------------
double fxid(int dt,double xt,double xd)
{
    double zeta[8]={0,      2.9690968,      0.40556079,     2.3253752,
                    1.1134924,  0.61194084,     1.485456392,    1.101493512};

    double F[5],xaid,xti=1.0/xt;
    if(xt>1.0&&xd>zeta[7]){
    	F[0]=-6.6047780;
        F[1]=8.9685638;
        F[2]=-0.87538506;
        F[3]=0.19264642;
        F[4]=1.94435064;
    }else{
    	F[0]=-1.4020567;
        F[1]=6.5384430;
        F[2]=0;
        F[3]=0;
        F[4]=1.5;
    }
    if(dt==0) xaid=F[0]*xt+F[1]+(F[2]+F[3]*xti)*xti+F[4]*log(xt);
    if(dt==1) xaid=F[0]+(F[4]-(F[2]+2.0*F[3]*xti)*xti)*xti;
    if(dt==2) xaid=(-F[4]+2.0*(F[2]+3.0*F[3]*xti)*xti)*xti*xti;

     return xaid;
}
//---------------------------------------------------------------------------
double fx(int dd,double xd,int dt,double xt)
{
    double zeta[8]={0,      2.9690968,      0.40556079,     2.3253752,
                    1.1134924,  0.61194084,     1.485456392,    1.101493512};
    double const_phi[5],p,q,xaJ;


    if(xt>1.0&&xd>zeta[7]){
        const_phi[1]=fxphi(0,zeta[1],1);
        const_phi[2]=fxphi(0,zeta[1],2);
        const_phi[3]=fxphi(0,zeta[1],3);
        const_phi[4]=fxphi(0,zeta[4],4)-fxphi(0,zeta[3],4);
        q=25.83;
    }else{
        const_phi[1]=0;
        const_phi[2]=0;
        const_phi[3]=0;
        const_phi[4]=fxphi(0,zeta[4],4)-fxphi(0,zeta[2],4);
        q=22.0;
    }

    if(dt==0){
        if(dd==0){
            xaJ=fxphi(0,xd,0)+
                xt*(fxphi(0,xd,1)-const_phi[1]+
                xt*(fxphi(0,xd,2)-const_phi[2]+
                xt*xt*(fxphi(0,xd,3)-const_phi[3])))+const_phi[4]*pow(xt,q);
        }
        if(dd==1){
            xaJ=fxphi(1,xd,0)+
                xt*(fxphi(1,xd,1)+xt*(fxphi(1,xd,2)+xt*xt*fxphi(1,xd,3)));
        }
        if(dd==2){
            xaJ=fxphi(2,xd,0)+
                xt*(fxphi(2,xd,1)+xt*(fxphi(2,xd,2)+xt*xt*fxphi(2,xd,3)));
        }
    }
    if(dt==1){
        if(dd==0){
            xaJ=fxphi(0,xd,1)-const_phi[1]+
                xt*(2.0*(fxphi(0,xd,2)-const_phi[2])+
                4.0*xt*xt*(fxphi(0,xd,3)-const_phi[3]))+
                q*const_phi[4]*pow(xt,q-1.0);
        }
        if(dd==1){
            xaJ=fxphi(1,xd,1)+
                xt*(2.0*fxphi(1,xd,2)+4.0*xt*xt*fxphi(1,xd,3));
        }
    }
    if(dt==2){
        xaJ=2.0*(fxphi(0,xd,2)-const_phi[2])+
                12.0*xt*xt*(fxphi(0,xd,3)-const_phi[3])+
                q*(q-1.0)*const_phi[4]*pow(xt,q-2.0);
    }

    xaJ+=fxphi4(dd,xd,dt,xt);
    if(xt<1.0/0.983&&xt>1.0/1.173&&xd>zeta[5]&&xd<zeta[6]) xaJ+=fxphicr(dd,xd,dt,xt);
    return xaJ;

}
//---------------------------------------------------------------------------
double fxphi4(int dd,double xd,int dt,double xt)
{
    double zeta[8]={0,      2.9690968,      0.40556079,     2.3253752,
                    1.1134924,  0.61194084,     1.485456392,    1.101493512};
    double xa4,p;

    if(xd>zeta[4]) p=25.83;
    else p=22.0;

    if(xd<zeta[2]){
        if(dd==0){
            if(dt==0) xa4=(fxphi(0,zeta[2],4)-fxphi(0,zeta[4],4))*pow(xt,p);
            if(dt==1) xa4=(fxphi(0,zeta[2],4)-fxphi(0,zeta[4],4))*p*pow(xt,p-1.0);
            if(dt==2)
                xa4=(fxphi(0,zeta[2],4)-fxphi(0,zeta[4],4))*p*(p-1.0)*pow(xt,p-2.0);
        }else xa4=0;

    }
    if(xd>zeta[3]){
        if(dd==0){
            if(dt==0) xa4=(fxphi(0,zeta[3],4)-fxphi(0,zeta[4],4))*pow(xt,p);
            if(dt==1) xa4=(fxphi(0,zeta[3],4)-fxphi(0,zeta[4],4))*p*pow(xt,p-1.0);
            if(dt==2)
                xa4=(fxphi(0,zeta[3],4)-fxphi(0,zeta[4],4))*p*(p-1.0)*pow(xt,p-2.0);
        }else xa4=0;
    }
    if(xd<zeta[3]&&xd>zeta[2]){
        if(dt==0){
            if(dd==0) xa4=(fxphi(0,xd,4)-fxphi(0,zeta[4],4))*pow(xt,p);
            if(dd==1) xa4=fxphi(1,xd,4)*pow(xt,p);
            if(dd==2) xa4=fxphi(2,xd,4)*pow(xt,p);
        }
        if(dt==1){
            if(dd==0) xa4=(fxphi(0,xd,4)-fxphi(0,zeta[4],4))*p*pow(xt,p-1.0);
            if(dd==1) xa4=fxphi(1,xd,4)*p*pow(xt,p-1.0);
        }
        if(dt==2){
            xa4=(fxphi(0,xd,4)-fxphi(0,zeta[4],4))*p*(p-1.0)*pow(xt,p-2.0);
        }
    }

    return xa4;
}
//---------------------------------------------------------------------------
double fxphicr(int dd,double xd,int dt,double xt)
{
    double zeta[8]={0,      2.9690968,      0.40556079,     2.3253752,
                    1.1134924,  0.61194084,     1.485456392,    1.101493512};
    double xacr;

    if(xd<zeta[7]){
        if(dd==0) xacr=fxphi(0,xd,5)-fxphi(0,zeta[5],5);
        if(dd==1) xacr=fxphi(1,xd,5);
        if(dd==2) xacr=fxphi(2,xd,5);
    }else{
        if(dd==0) xacr=fxphi(0,xd,6)-fxphi(0,zeta[6],6);
        if(dd==1) xacr=fxphi(1,xd,6);
        if(dd==2) xacr=fxphi(2,xd,6);
    }

    if(dt==0) xacr*=1.46*pow(xt,251.0)+1.02*pow(xt,22.0);
    if(dt==1) xacr*=251.0*1.46*pow(xt,250.0)+22.0*1.02*pow(xt,21.0);
    if(dt==2) xacr*=251.0*250.0*1.46*pow(xt,249.0)+22.0*21.0*1.02*pow(xt,20.0);

    return xacr;
}
//---------------------------------------------------------------------------
double fxphi(int dd,double xd,int idx)
{

	double F[7][10]={

            {0,
            0.48176287,    -0.22996818,    0.67678089,     -0.82687853,
            0.59684665,     -0.26046265,    0.068249584,    -0.0098938026,
            0.00060938476},

            {0,
            -1.17810687,   0.611458185,    -1.59127084,    2.18816951,
            -1.66416239,    0.741574506,    -0.192234256,   0.0268815303,
            -0.00156464723},

            {0,
            -0.24051263,   -0.357633417,   1.38545921,     -2.2662842,
            1.89132269,     -0.896367224,   0.244259366,    -0.0356830831,
            0.00216704074},

            {0,
            -0.0750908703, -0.0394741659,  -0.0376038501,  0.404244668,
            -0.462622543,   0.24081461,     -0.0668988147,  0.00967590594
            -0.000575094683},

            {0,
            -0.213826595,  0.853137033,    -1.77422585,    2.11856977,
            -1.53874755,    0.693737872,    -0.190153915,   0.0290866982,
            -0.00190789666},

            {0,
            -0.00701074245,    0.03926238030,  -0.110956774,   0.175796531
            -0.158135317,   0.0752434365,   -0.0146753994,  0,0},

            {0,
            -3.56560121,   8.73657614, -11.83502506,   9.56797096,
            -4.6146467,     1.22917766, -0.13948161,    0,0}
		};


    double xa0=0,xdt=1.0;
    if(dd==0) for(int i=0;i<=9;i++,xdt*=xd) xa0+=F[idx][i]*xdt;
    if(dd==1) for(int i=1;i<=9;i++,xdt*=xd) xa0+=F[idx][i]*xdt*i;
    if(dd==2) for(int i=2;i<=9;i++,xdt*=xd) xa0+=F[idx][i]*xdt*i*(i-1);

    return xa0;
}
//---------------------------------------------------------------------------
void fxsatu(double xt,AnsiString *axp,AnsiString *axdg,AnsiString *axdl)
{

	if(xt<=1.0){
    	*axp=*axdg=*axdl="over Tc";
    	return;
    }

	double xp=0,xdg=0,xdl=0,xta=1.0-1.0/xt;

	double Np[]={0,-5.988397,  1.358585,   -0.9018093,     -1.343456e2};
    xp=pc*exp(xta*xt*(Np[1]+sqrt(xta)*(Np[2]+Np[3]*xta)+Np[4]*pow(xta,11.0)));


	double Nv[]={0.99942,-5.7286,42.909,-305.78,1350.6,
        -3557.7,5471.1,-4525.7,1554.1};

    double thx=sqrt(xta),thxe=1;
    for(int i=0;i<9;i++,thxe*=thx) xdg+=Nv[i]*thxe;
                                //this is a function the author fitted.

	double Nl[]={
    	0.5336001,			2.461331,			-0.9772429,
        0.7076088
        };

    thx=sqrt(thx);  //thx=sqrt(sqrt(1-xt))
    xdl=1+thx*(Nl[0]+thx*(Nl[1]+thx*(Nl[2]+xta*Nl[3])));
    			//1st-expected satu-rho-liq (scaled by ds)


   	double drx,ddr,px=xp,px0;

	do{

        px0=px;

	    drx=xdg/1.95,ddr=xdg/4;

		do{
    		px=drx*ds*R*T0*fxZ(drx,xt);
    	    if(px>px0) drx-=ddr;
    	    else drx+=ddr;
    	    ddr/=2;
    	    if(drx<0) break;
    	}while(ddr/drx>=ALM0);
        xdg=drx;

   		drx=(3.2+xdl)/2,ddr=(drx-xdl)/2.8;

        do{
    		px=drx*ds*R*T0*fxZ(drx,xt);
    	    if(px>px0) drx-=ddr;
    	    else drx+=ddr;
    	    ddr/=2;
    	    if(drx<0) break;
        }while(ddr/drx>=ALM0);
        xdl=drx;

        px=ds*R*T0*xdg*xdl/(xdl-xdg)*(log(xdl/xdg)+fx(0,xdl,0,xt)-fx(0,xdg,0,xt));


	}while(fabs(px-px0)/px>=ALM0);

   	*axp=AnsiString(px);
    *axdg=AnsiString(xdg*dc);
    *axdl=AnsiString(xdl*dc);

}
//---------------------------------------------------------------------------
AnsiString fxmeltp(double xt)
{
	return "Not treat.";
}
//---------------------------------------------------------------------------
AnsiString fxrangeT(double xt)
{
	if(xt<161.36)
		return "Not applied to the range less than Tt (161.36 K).";
	if(xt>800.0)
		return "Not applied to the range more than 800 K.";
    return "";
}
//---------------------------------------------------------------------------
AnsiString fxrangePT(double xp,double xt)
{
	if(Mp!=0&&xp>Mp)
		return "Suppose to freeze.";
	if(xp<0)
		return "This density is not adequate.";
	if(xp>350e6)
		return "Not applied to the range more than 350 MPa.";

	return "";
}
//---------------------------------------------------------------------------
double fxZ(double xd,double xt,double xp)
{
	return 1+xd*fx(1,xd,0,xt);
}
//---------------------------------------------------------------------------
void fxX(double xd,double xt,double xp,
		double *xxF,double *xxU,double *xxH,double *xxG,double *xxS,
        double *xxCv,double *xxCp,double *xxZ,double *xxFP,
        double *xxW,double *xxJT,double *xxC2,double *xxAp,double *xxkT)
{

        double helm=fx(0,xd,0,xt)+fxid(0,xt,xd)+log(xd),
        		helmT1=xt*(fx(0,xd,1,xt)+fxid(1,xt,xd)),
                helmT2=xt*xt*(fx(0,xd,2,xt)+fxid(2,xt,xd)),
                helmd1=xd*fx(1,xd,0,xt),
                helmd1T1=xd*xt*fx(1,xd,1,xt),
                helmd2=xd*xd*fx(2,xd,0,xt);

        *xxZ=1+helmd1;
        double RT0=R*T0;

        *xxF=RT0*helm;
        *xxU=RT0*helmT1;
		*xxH=RT0*(*xxZ+helmT1);
		*xxG=RT0*(*xxZ+helm);
        *xxS=R*(helmT1-helm);
        *xxCv=-R*helmT2;

        double ZdT=*xxZ-helmd1T1,ZdT2=2*(*xxZ)-1+helmd2;

        *xxCp=R*(-helmT2+ZdT*ZdT/ZdT2);

		*xxFP=exp(-log(*xxZ)+*xxZ-1+fx(0,xd,0,xt));

		*xxW=sqrt(RT0/Mw*1000*(ZdT2-ZdT*ZdT/helmT2));
        *xxJT=-(*xxZ-1+helmd2+helmd1T1)/(ZdT*ZdT-helmT2*ZdT2)/(R*d0);
        *xxC2=0;//fx(0,0,-2,xt)/ds;
        *xxAp=ZdT/(T0*ZdT2);
        *xxkT=1.0/(d0*RT0*ZdT2);
}
//---------------------------------------------------------------------------


